"""Implements badges check."""

from pathlib import Path

from pyvelocity.checks import Check
from pyvelocity.checks import Result
from pyvelocity.configurations.files.readme import ReadMe


class Badges(Check):
    """Checks that README.md contains expected badges."""

    ID = "badges"

    def execute(self) -> Result:
        readme_path = Path("README.md")
        if not readme_path.exists():
            return Result(self.ID, is_ok=False, message="README.md file not found")

        readme = ReadMe(readme_path)
        missing_badges = self._get_missing_badges(readme)

        if missing_badges:
            message = f"README.md is missing the following badges: {', '.join(missing_badges)}"
            return Result(self.ID, is_ok=False, message=message)

        return Result(self.ID, is_ok=True, message="")

    def _get_missing_badges(self, readme: ReadMe) -> list[str]:
        expected_badges = [
            ("Test badge", readme.has_test_badge()),
            ("CodeQL badge", readme.has_codeql_badge()),
            ("Code Coverage badge", readme.has_coverage_badge()),
            ("Maintainability badge", readme.has_maintainability_badge()),
            ("Dependabot badge", readme.has_dependabot_badge()),
            ("Python versions badge", readme.has_python_versions_badge()),
            ("Social badge", readme.has_social_badge()),
        ]

        missing_badges = []
        for badge_name, has_badge in expected_badges:
            if not has_badge:
                missing_badges.append(badge_name)

        return missing_badges



"""GitHub-specific regex patterns for PyVelocity."""

import re
import urllib.parse

from pyvelocity.regex.markdown import RegexPatternsMarkDown

GITHUB_BASE_URL = "https://github.com/"


class RegexPatternsGitHub:
    """GitHub-specific regex patterns for badges and URLs."""

    USER = r"[a-zA-Z0-9_.-]+"
    REPOSITORY = r"[a-zA-Z0-9_.-]+"
    REPOSITORY_URL = re.escape(GITHUB_BASE_URL) + rf"{USER}/{REPOSITORY}"
    BASE_URL_ENCODED = (
        re.escape(urllib.parse.quote_plus(GITHUB_BASE_URL))
        + USER
        + re.escape(urllib.parse.quote_plus("/"))
        + REPOSITORY
    )

    @classmethod
    def workflow_badge(cls, workflow_name: str) -> str:
        name = workflow_name
        return RegexPatternsMarkDown.badge_pattern(name, cls.url_workflow_badge(name), cls.url_queried_actions(name))

    @classmethod
    def url_workflow_badge(cls, workflow_name: str) -> str:
        return rf"{cls.REPOSITORY_URL}/workflows/{workflow_name}/badge\.svg"

    @classmethod
    def url_queried_actions(cls, workflow_name: str) -> str:
        return rf"{cls.REPOSITORY_URL}/actions\?query=workflow%3A{workflow_name}"

    @classmethod
    def dependabot_badge(cls) -> str:
        return RegexPatternsMarkDown.badge_pattern(
            "Dependabot",
            rf"https://flat\.badgen\.net/github/dependabot/{cls.USER}/{cls.REPOSITORY}\?icon=dependabot",
            rf"{cls.REPOSITORY_URL}/security/dependabot",
        )



"""Qlty-specific regex patterns for PyVelocity."""

from pyvelocity.regex.github import RegexPatternsGitHub
from pyvelocity.regex.markdown import RegexPatternsMarkDown


class RegexPatternsQlty:
    """Qlty-specific regex patterns for code quality badges."""

    PROJECT_ID = r"[a-zA-Z0-9_.-]+"
    URL_PROJECT_PAGE = (
        rf"https://qlty\.sh/gh/{RegexPatternsGitHub.USER}/{RegexPatternsGitHub.REPOSITORY}/projects/{PROJECT_ID}"
    )
    URL_COVERAGE_BADGE = rf"{URL_PROJECT_PAGE}/coverage\.svg"
    URL_MAINTAINABILITY_BADGE = rf"{URL_PROJECT_PAGE}/maintainability\.svg"

    @classmethod
    def coverage_badge(cls) -> str:
        return RegexPatternsMarkDown.badge_pattern("Code Coverage", cls.URL_COVERAGE_BADGE, cls.URL_PROJECT_PAGE)

    @classmethod
    def maintainability_badge(cls) -> str:
        return RegexPatternsMarkDown.badge_pattern(
            "Maintainability",
            cls.URL_MAINTAINABILITY_BADGE,
            cls.URL_PROJECT_PAGE,
        )



"""Markdown and general regex patterns for PyVelocity."""


class RegexPatternsMarkDown:
    """Collection of reusable regex patterns used throughout the codebase."""

    # Python version specification patterns
    GREATER_EQUAL_VERSION = r">=(\d+)\.(\d+)"
    LESS_THAN_VERSION = r"<(\d+)\.(\d+)"
    COMPATIBLE_RELEASE_VERSION = r"~=(\d+)\.(\d+)"
    EXACT_VERSION = r"^(\d+)\.(\d+)$"

    # Python classifier patterns
    PYTHON_VERSION_CLASSIFIER = r"^Programming Language :: Python :: (\d+\.\d+)$"

    @staticmethod
    def badge_pattern(alt_text: str, image_url_pattern: str, link_url_pattern: str) -> str:
        """Generate a regex pattern to match markdown badge format.

        Args:
            alt_text: The alt text for the badge (e.g., "Test", "CodeQL")
            image_url_pattern: Regex pattern for the badge image URL
            link_url_pattern: Regex pattern for the badge link URL

        Returns:
            Complete regex pattern to match the badge in markdown format
        """
        return rf"\[!\[{alt_text}\]\({image_url_pattern}\)\]\({link_url_pattern}\)"



"""PyPI-specific regex patterns for PyVelocity."""

import re
import urllib.parse

from pyvelocity.regex.markdown import RegexPatternsMarkDown

PYPI_BASE_URL = "https://pypi.org/project/"


class RegexPatternsPyPI:
    """PyPI-specific regex patterns for packages and badges."""

    PACKAGE_NAME = r"[a-zA-Z0-9_.-]+"
    BASE_URL = re.escape(PYPI_BASE_URL) + PACKAGE_NAME
    BASE_URL_ENCODED = (
        re.escape(urllib.parse.quote_plus(PYPI_BASE_URL)) + PACKAGE_NAME + re.escape(urllib.parse.quote_plus("/"))
    )
    SHIELDS_PYVERSIONS_URL = rf"https://img\.shields\.io/pypi/pyversions/{PACKAGE_NAME}(?:\.svg)?"

    @classmethod
    def python_versions_badge(cls) -> str:
        return RegexPatternsMarkDown.badge_pattern(
            "Python versions",
            cls.SHIELDS_PYVERSIONS_URL,
            rf"{cls.BASE_URL}/?/?",
        )



"""Implements README.md file."""

import re
from pathlib import Path

from pyvelocity.configurations.files import ConfigurationFile
from pyvelocity.regex.github import RegexPatternsGitHub
from pyvelocity.regex.pypi import RegexPatternsPyPI
from pyvelocity.regex.qlty import RegexPatternsQlty
from pyvelocity.regex.shieldsio import RegexPatternsShieldsIO

WHERE_README_MD = "README.md"


class ReadMe(ConfigurationFile):
    """README.md file."""

    def __init__(self, path_readme: Path) -> None:
        super().__init__()
        self.content = path_readme.read_text(encoding="utf-8") if path_readme.exists() else ""

    @property
    def name(self) -> str:
        return WHERE_README_MD

    def has_badge(self, badge_text: str) -> bool:
        """Check if README contains a specific badge text."""
        return badge_text in self.content

    def has_test_badge(self) -> bool:
        """Check if README contains Test badge with strict markdown format."""
        return bool(re.compile(RegexPatternsGitHub.workflow_badge("Test")).search(self.content))

    def has_codeql_badge(self) -> bool:
        """Check if README contains CodeQL badge with strict markdown format."""
        return bool(re.compile(RegexPatternsGitHub.workflow_badge("CodeQL")).search(self.content))

    def has_coverage_badge(self) -> bool:
        """Check if README contains Code Coverage badge with strict markdown format."""
        return bool(re.compile(RegexPatternsQlty.coverage_badge()).search(self.content))

    def has_maintainability_badge(self) -> bool:
        """Check if README contains Maintainability badge with strict markdown format."""
        return bool(re.compile(RegexPatternsQlty.maintainability_badge()).search(self.content))

    def has_dependabot_badge(self) -> bool:
        """Check if README contains Dependabot badge with strict markdown format."""
        return bool(re.compile(RegexPatternsGitHub.dependabot_badge()).search(self.content))

    def has_python_versions_badge(self) -> bool:
        """Check if README contains Python versions badge with strict markdown format."""
        return bool(re.compile(RegexPatternsPyPI.python_versions_badge()).search(self.content))

    def has_social_badge(self) -> bool:
        """Check if README contains social/sharing badge with strict markdown format."""
        return bool(re.compile(RegexPatternsShieldsIO.x_badge()).search(self.content))



"""Shields.io-specific regex patterns for PyVelocity."""

from pyvelocity.regex.github import RegexPatternsGitHub
from pyvelocity.regex.markdown import RegexPatternsMarkDown
from pyvelocity.regex.pypi import RegexPatternsPyPI


class RegexPatternsShieldsIO:
    """Shields.io-specific regex patterns for social media badges."""

    @classmethod
    def x_badge(cls) -> str:
        return RegexPatternsMarkDown.badge_pattern(
            "X URL",
            rf"https://img\.shields\.io/twitter/url\?style=social&url={RegexPatternsGitHub.BASE_URL_ENCODED}",
            rf"https://x\.com/intent/post\?text=[^&]*&url={RegexPatternsPyPI.BASE_URL_ENCODED}&hashtags=python",
        )
